import Vue from 'vue'

// initial state
export const state = {
  entries: []
}

const blockingVirtualFlag = flags => {
  const anyBlockingFlags = flags.filter(f => f.isBlocking).filter(f => f.isActive).length > 0
  const value = 'blocking'
  const name = 'Blocking Flag(s)'
  const type = 'Process'
  const now = new Date()
  return anyBlockingFlags
    ? { value, name, type, isActive: true, isSet: true, isCleared: false, dateTimeSet: now, setBy: '', isBlocking: true }
    : { value, name, type, isActive: false, isSet: false, isCleared: false, isBlocking: true }
}

const endorseVirtualFlag = flags => {
  // Get new endorsement flag data
  const endorsementNewFlag = flags.filter(f => f.value === 'endorsement')
  if(endorsementNewFlag.length > 0) return endorsementNewFlag[0]

  // Maintain old functionality
  const edFlag = flags.filter(f => f.value === 'ED').filter(f => f.isActive)
  const eiFlag = flags.filter(f => f.value === 'EI').filter(f => f.isActive)
  const allFlags = edFlag.concat(eiFlag)
  if (allFlags.length > 0) return allFlags[0]

  // Default if no flag data is found
  const value = 'endorsement'
  const name = 'Endorsement Problem'
  const type = 'Process'
  return { value, name, type, isActive: false, isSet: false, isCleared: false, isBlocking: true, state: 'indeterminate' }
}

const cesHoldsVirtualFlag = flags => {
  const cesHoldsFlag = flags.filter(f => f.value === 'cesHolds')
  if (cesHoldsFlag.length > 0) return cesHoldsFlag[0]

  const cesFlags = flags.find(f => f.value === 'CES')
  if (cesFlags != null) {
    return { ...cesFlags, state: 'fail' }
  }

  const value = 'cesHolds'
  const name = 'CES Standard Hold'
  const type = 'Process'
  return { value, name, type, isActive: false, isSet: false, isCleared: false, isBlocking: true, state: 'indeterminate' }
}

// getters
export const getters = {
  flags (state) {
    console.log('getter flags', state)
    const { entries = [] } = state
    return entries
  },
  flagByValue: (state, getters) => flagValue => {
    const flags = getters.flags
    if (flagValue === 'blocking') {
      return blockingVirtualFlag(flags)
    } else if (flagValue === 'endorsement') {
      return endorseVirtualFlag(flags)
    }  else if (flagValue === 'cesHolds') {
      return cesHoldsVirtualFlag(flags)
    }
    const flag = flags.find(({ value }) => value === flagValue)
    return flag || { value: flagValue, name: flagValueToName(flagValue), isSet: false, isCleared: false, isActive: false }
  }
}

// actions
export const actions = {
}

// mutations
export const mutations = {
  resetFlags (state) {
    state.entries = []
  },
  setFlags (state, flags = []) {
    state.entries = flags.map(convertFlag)
  },
  updateFlag (state, flag) {
    const index = state.entries.findIndex(item => item.value === flag.value)
    if (index < 0) {
      state.entries.push(convertFlag(flag))
    } else {
      Vue.set(state.entries, index, convertFlag(flag))
    }
  }
}

export const convertFlag = flag => {
  if (!flag) return {}
  const clearedIsAfterSet = ({ dateTimeSet, dateTimeCleared }) => {
    if (!dateTimeCleared) return false
    const clearedAsDate = new Date(dateTimeCleared)
    const setAsDate = new Date(dateTimeSet)
    return clearedAsDate >= setAsDate
  }
  const isSet = !!flag.dateTimeSet
  const isCleared = isSet ? clearedIsAfterSet(flag) : false
  const isActive = isSet && !isCleared
  const {
    name,
    value,
    type,
    dateTimeSet,
    setBy,
    dateTimeCleared,
    clearedBy,
    state,
    message
  } = flag
  const isBlocking = /process|wait/i.test(type)
  return {
    value,
    name,
    type,
    isActive,
    isSet,
    isCleared,
    dateTimeSet,
    setBy,
    dateTimeCleared,
    clearedBy,
    isBlocking,
    state,
    message
  }
}

export const flagValueToName = value => {
  switch (value) {
    case 'AT':
      return 'Applicant Type Mismatch'
    case 'BGCR':
      return 'Background Criminal Record Issue'
    case 'BGPM':
      return 'Background Plural Marriage Issue'
    case 'BGSR':
      return 'Background Sex Registry Issue'
    case 'BGUD':
      return 'Background University Discipline Issue'
    case 'BI':
      return 'Background Issues'
    case 'BYUAPP':
      return 'BYU Application'
    case 'CAENSIGN':
      return 'Concurrent Ensign College Application'
    case 'CAIDAHO':
      return 'Concurrent BYU Idaho Application'
    case 'CDND':
      return 'Counselor Do Not Deny'
    case 'CES':
      return 'CES Standard Hold'
    case 'CHLD':
      return 'Counselor Hold'
    case 'ED':
      return 'Endorsement Denied'
    case 'EI':
      return 'Endorsement Issues'
    case 'EP':
      return 'Endorsement Pending'
    case 'EL':
      return 'English Language Issues'
    case 'HI':
      return 'Hawaii Applicants'
    case 'IA':
      return 'International Applicants'
    case 'IW':
      return 'IWork Applicants'
    case 'INC':
      return 'Incomplete Application'
    case 'LA':
      return 'Low Academics'
    case 'LACH':
      return 'Low Academics College GPA'
    case 'LAHS':
      return 'Low Academics High School GPA'
    case 'LATS':
      return 'Low Academics Test Score'
    case 'NFNG':
      return 'NF Not Graduating'
    case 'NOHS':
      return 'Under 18 No High School Completion'
    case 'NSE':
      return 'No Secondary Education'
    case 'PD':
      return 'Previously Denied'
    case 'UA':
      return 'Underage Applicant'
    case 'UNASSIGNED':
      return 'Unassigned Applicant'
    case 'US':
      return 'Domestic Applicants'
    default:
      return 'Assigned'
  }
}
